# -*- coding: utf-8 -*-

import uuid

from django.db import models
from django.contrib.auth.models import AbstractBaseUser
from django.core.exceptions import ValidationError as DjangoValidationError
from django.test import TestCase, override_settings

from mongoengine import ValidationError
from timberjack.fields import ContentType, ContentTypeField, UserPKField


def integer_validator(value):
    if not isinstance(value, int):
        raise DjangoValidationError('%(1)r is not an integer.',
                                    code='invalid',
                                    params={'value': value})


def uuid_validator(value):
    try:
        uuid.UUID(value)
    except ValueError:
        raise DjangoValidationError('%(value)s is not a valid UUID.',
                                    code='invalid',
                                    params={'value': value})


class ContentTypeFieldTestCase(TestCase):

    def setUp(self):
        self.field = ContentTypeField()

    def test_regex(self):
        self.assertIsNone(self.field.regex.match('auth'))
        self.assertIsNone(self.field.regex.match('auth user'))

        app_label, model = self.field.regex.match('auth.user').groups()
        self.assertEqual(app_label, 'auth')
        self.assertEqual(model, 'user')

        app_label, model = self.field.regex.match('auth.user.should.not.match').groups()
        self.assertEqual(app_label, 'auth')
        self.assertEqual(model, 'user')

        try:
            self.field.regex.match('auth').groups()
            self.fail('Did not fail when trying to call method on non-matched result')
        except AttributeError as e:
            self.assertEqual(str(e), "'NoneType' object has no attribute 'groups'")

    def test_validate_ctype_string(self):
        self.assertIsNone(self.field.validate('auth.user'))

    def test_validate_ctype_instance(self):
        ctype = ContentType.objects.get(app_label='auth', model='user')
        self.assertIsNone(self.field.validate(ctype))
        
    def test_validate_invalid_ctype_string(self):
        try:
            self.field.validate('invalid')
            self.fail('Did not fail when validating invalid content type string.')
        except ValidationError as e:
            self.assertEqual(str(e), 'Could not look up ContentType object for "invalid".')

    def test_validate_non_existent_ctype(self):
        try:
            self.field.validate('invalid.model')
            self.fail('Did not fail when trying to look up non-existent ContentType.')
        except ValidationError as e:
            self.assertEqual(str(e), 'Could not look up ContentType object for "invalid.model".')

    def test_ctype_string_to_python(self):
        self.assertEqual(
            self.field.to_python('auth.user'),
            ContentType.objects.get(app_label='auth', model='user'),
        )

    def test_ctype_string_to_mongo(self):
        self.assertEqual(self.field.to_mongo('auth.user'), 'auth.user')

    def test_ctype_instance_to_mongo(self):
        self.assertEqual(
            self.field.to_mongo(ContentType.objects.get(app_label='auth', model='user')),
            'auth.user'
        )


@override_settings(AUTH_USER_MODEL='timberjack.NoPKFieldUserModel')
class NoPKFieldUserTestCase(TestCase):
    """
    Test that a user model with no defined primary key works.
    """
    class NoPKFieldUserModel(AbstractBaseUser):
        username = models.CharField(max_length=255)
        USERNAME_FIELD = 'username'

        class Meta:
            app_label = 'timberjack'

    def setUp(self):
        self.field = UserPKField()

    def test_pk_field_is_autofield(self):
        self.assertIsInstance(self.field.pk_field, models.AutoField)

    def test_validate_integer(self):
        self.assertIsNone(self.field.validate(1))


@override_settings(AUTH_USER_MODEL='timberjack.IntegerUserModel')
class IntegerUserPKFieldTestCase(TestCase):
    """
    Test that a user model with a custom integer based primary key works.
    """
    class IntegerUserModel(AbstractBaseUser):
        id = models.IntegerField(primary_key=True, unique=True, validators=[integer_validator])
        username = models.CharField(max_length=255)
        USERNAME_FIELD = 'username'

        class Meta:
            app_label = 'timberjack'

    def setUp(self):
        self.field = UserPKField()

    def test_validate_integer(self):
        self.assertIsNone(self.field.validate(1))

    def test_validate_string_fails(self):
        try:
            self.field.validate('this is not an integer!')
            self.fail('Did not fail when trying to validate a string.')
        except ValidationError as e:
            self.assertEqual(str(e), "'this is not an integer!' value must be an integer.")

    def test_value_conversion(self):
        value = 1
        mongo_val = self.field.to_mongo(value)
        python_val = self.field.to_python(mongo_val)
        self.assertEqual(value, python_val)


@override_settings(AUTH_USER_MODEL='timberjack.StringUserModel')
class StringUserPKFieldTestCase(TestCase):
    """
    Test that a user model with a custom string based primary key works.
    """
    class StringUserModel(AbstractBaseUser):
        id = models.CharField(primary_key=True, unique=True, max_length=6)
        username = models.CharField(max_length=255)
        USERNAME_FIELD = 'username'

        class Meta:
            app_label = 'timberjack'

    def setUp(self):
        self.field = UserPKField()

    def test_validate_string(self):
        self.assertIsNone(self.field.validate('unique'))

    def test_validate_too_long_string_fails(self):
        try:
            self.field.validate('this string is more than 6 characters long!')
            self.fail('Did not fail when trying to validate a too long string.')
        except ValidationError as e:
            self.assertEqual(str(e), 'Ensure this value has at most 6 characters (it has 43).')

    def test_validate_int_fails(self):
        try:
            self.field.validate(1)
            self.fail('Did not fail when trying to validate a non string type.')
        except ValidationError as e:
            self.assertEqual(str(e), 'Value %(value)r is not a valid primary key value for '
                                     '<django.db.models.fields.CharField: id>.' % {'value': 1})

    def test_value_conversion(self):
        value = 'value'
        mongo_val = self.field.to_mongo(value)
        python_val = self.field.to_python(mongo_val)
        self.assertEqual(value, python_val)


@override_settings(AUTH_USER_MODEL='timberjack.UUIDUserModel')
class UUIDUserPKFieldTestCase(TestCase):
    """
    Test that a user model with a custom UUID based primary key works.
    """
    class UUIDUserModel(AbstractBaseUser):
        id = models.UUIDField(primary_key=True, unique=True, validators=[uuid_validator])
        username = models.CharField(max_length=255)
        USERNAME_FIELD = 'username'

        class Meta:
            app_label = 'timberjack'

    def setUp(self):
        self.field = UserPKField()

    def test_validate_uuid_string(self):
        self.assertIsNone(self.field.validate('6a3de76f-ebd6-448d-a70f-62e0866d37f4'))

    def test_validate_uuid_hex_string(self):
        self.assertIsNone(self.field.validate('74a40ec6a74d44199d8457861bb51484'))

    def test_validate_non_valid_uuid_fails(self):
        try:
            self.field.validate('this-is-not-an-uuid')
            self.fail('Did not fail when trying to validate a non-valid uuid string.')
        except ValidationError as e:
            self.assertEqual(str(e), "'this-is-not-an-uuid' is not a valid UUID.")

    def test_value_conversion(self):
        value = uuid.uuid4()
        mongo_val = self.field.to_mongo(value)
        python_val = self.field.to_python(mongo_val)
        self.assertEqual(value, python_val)



# -*- coding: utf-8 -*-

import re
import operator
from functools import reduce

from django.core.exceptions import ValidationError
from django.contrib.auth import get_user_model
from django.contrib.contenttypes.models import ContentType
from django.utils.translation import ugettext_lazy as _

from mongoengine import fields

_ctype_cache = {}


class ContentTypeField(fields.StringField):
    """
    Store django ContentType references.
    """
    regex = re.compile(r'^(\w+)\.(\w+)')

    def __init__(self, *args, **kwargs):
        super(ContentTypeField, self).__init__(regex=self.regex, *args, **kwargs)

    def to_python(self, value):
        if isinstance(value, ContentType):
            ctype_string = '{app_label}.{model}'.format(app_label=value.app_label, model=value.model)
            if ctype_string not in _ctype_cache:
                _ctype_cache[ctype_string] = value
            return value

        elif isinstance(value, str):
            try:
                return _ctype_cache[value]
            except KeyError:
                app_label, model = value.split('.', 1)
                ctype_string = '{app_label}.{model}'.format(app_label=app_label, model=model)
                if ctype_string not in _ctype_cache:
                    _ctype_cache[ctype_string] = ContentType.objects.get(app_label=app_label, model=model)
            finally:
                return _ctype_cache[value]

    def to_mongo(self, value):
        if isinstance(value, str):
            return '.'.join(self.regex.match(value).groups())
        if isinstance(value, ContentType):
            return '{app_label}.{model}'.format(app_label=value.app_label, model=value.model)

    def validate(self, value):
        if isinstance(value, str):
            try:
                app_label, model = self.regex.match(value).groups()
                value = '{app_label}.{model}'.format(app_label=app_label, model=model)
                if value not in _ctype_cache:
                    _ctype_cache.update({value: ContentType.objects.get(app_label=app_label, model=model)})
            except (AttributeError, ContentType.DoesNotExist):
                self.error('Could not look up ContentType object for "%s".' % value)

        if isinstance(value, ContentType):
            value = '{app_label}.{model}'.format(app_label=value.app_label, model=value.model)

        super(ContentTypeField, self).validate(value)


class UserPKField(fields.DynamicField):
    """
    Dynamic field which piggybacks on the settings.AUTH_USER_MODEL
    validation and value converting.
    """

    default_error_messages = {
        'non_field_error': _('Value %(value)r is not a valid primary key value for %(field)r.')
    }

    @property
    def pk_field(self):
        model = get_user_model()
        pk_field = reduce(operator.eq,
                          filter(lambda field: field.primary_key, model._meta.fields))
        return pk_field

    def to_python(self, value):
        return self.pk_field.to_python(value)

    def to_mongo(self, value, **kwargs):
        # NOTE: This is probably way to naive!
        return self.to_python(value)

    def validate(self, value, clean=True):
        try:
            self.pk_field.run_validators(value)
        except ValidationError as e:
            if hasattr(e, 'code') and e.code in self.pk_field.error_messages:
                e.message = self.pk_field.error_messages[e.code]
            message = getattr(e, 'message', '. '.join(e.messages))
            self.error(message)
        except Exception:
            # All other exceptions are recorded as 'non_field_error'.
            message = self.default_error_messages['non_field_error'] % {
                'value': value,
                'field': self.pk_field
            }
            self.error(message)




# -*- coding: utf-8 -*-

from django.core.exceptions import ValidationError as DjangoValidationError
from django.core.validators import validate_ipv46_address

from mongoengine import ValidationError


def validate_ip_address(value):
    """
    Make sure value is a valid IPv4 or IPv6 address.
    """
    try:
        validate_ipv46_address(value)
        return value
    except DjangoValidationError as e:
        raise ValidationError(str(e))



# -*- coding: utf-8 -*-

from django.apps import apps
from django.core.exceptions import FieldDoesNotExist
from django.db import models
from django.forms import modelform_factory, ALL_FIELDS
from timberjack.constants import CREATE, READ, UPDATE, DELETE


class MethodActionMap(object):
    """
    Class which maps HTTP methods to user actions.
    """
    action_map = {
        "GET": READ,
        "POST": CREATE,
        "PUT": UPDATE,
        "PATCH": UPDATE,
        "DELETE": DELETE,
    }

    def __init__(self, request):
        self._method = request.method

    @property
    def method(self):
        return self.action_map.get(self._method, None)


class BaseObjectAccessLogMixin(object):
    """
    Some docstring here..
    """

    method_action_map_class = MethodActionMap

    def get_method_action_map_class(self):
        assert self.method_action_map_class is not None, (
            "'%s' should either include a `method_action_map_class` attribute, "
            "or override the `get_method_action_map_class()` method."
            % self.__class__.__name__
        )
        return self.method_action_map_class

    def get_method_action(self, request):
        method_action_class = self.get_method_action_map_class()
        return method_action_class(request).method

    def get_form(self, request, obj):
        return modelform_factory(apps.get_model(obj._meta.app_label, obj._meta.model_name),
                                 fields=ALL_FIELDS)

    def construct_message(self, request, obj=None):
        """
        Constructs a log message for the requested object. Optionally,
        pass a custom form used to figure out changed data.
        """
        action_flag = self.get_method_action(request)

        ModelForm = self.get_form(request, obj)
        if request.method == 'POST':
            form = ModelForm(request.POST, request.FILES, instance=obj)
        else:
            # NOTE: Do we need to care about this one?
            if action_flag is CREATE:
                initial = dict(request.GET.items())
                for key in initial:
                    try:
                        field = obj._meta.model._meta.get_field(key)
                    except FieldDoesNotExist:
                        continue
                    if isinstance(field, models.ManyToManyField):
                        initial[key] = initial[key].split(',')
                form = ModelForm(initial=initial)
            else:
                form = ModelForm(instance=obj)

        message = []
        if action_flag is CREATE:
            message.append({'created': {}})
        elif action_flag is READ:
            message.append({'read': {}})
        elif action_flag is UPDATE:
            message.append({'updated': {'fields': form.changed_data}})
        elif action_flag is DELETE:
            message.append({'deleted': {}})

        return message



# -*- coding: utf-8 -*-

from django.conf.urls import url
from django.contrib.auth.models import User
from django.core.urlresolvers import reverse
from django.test import TestCase, override_settings

from django.views.generic import CreateView, UpdateView, DeleteView, DetailView


class CreateUserView(CreateView):
    model = User
    fields = '__all__'


class UpdateUserView(UpdateView):
    model = User
    fields = '__all__'


class DeleteUserView(DeleteView):
    model = User


class DetailUserView(DetailView):
    model = User
    fields = '__all__'


urlpatterns = [
    url(r'^create-user/$', view=CreateUserView.as_view(), name='create-user'),
    url(r'^update-user/(?P<pk>\d+)/$', view=UpdateUserView.as_view(), name='update-user'),
    url(r'^delete-user/(?P<pk>\d+)/$', view=DeleteUserView.as_view(), name='delete-user'),
    url(r'^detail-user/(?P<pk>\d+)/$', view=DetailUserView.as_view(), name='detail-user')
]


@override_settings(ROOT_URLCONF='tests.test_django_forms_mixin')
class AccessLogModelFormMixinTestCase(TestCase):

    def setUp(self):
        self.user = User.objects.create_user('testuser', 'testuser@example.com', 'test123.')
        self.client.login(username=self.user.username, password='test123.')

    def test_create_view_object_is_logged(self):
        response = self.client.post(reverse('create-user'), data={
            'username': 'another-user',
            'email': 'another-user@example.com',
            'password': 'test123.'
        })
        self.assertEqual(response.status_code, 200)

    def test_detail_view_object_is_logged(self):
        response = self.client.get(reverse('detail-user', kwargs={'pk': self.user.pk}))
        self.assertEqual(response.status_code, 200)



# -*- coding: utf-8 -*-

from django.apps import AppConfig

from . import connections


class TimberJackConfig(AppConfig):  # pragma: no cover
    name = 'timberjack'
    label = 'timberjack'

    def ready(self):
        """
        Reload all mongo connections.
        """
        connections.reload_all()



# -*- coding: utf-8 -*-

from django.test import TestCase
from mongoengine import ValidationError
from timberjack.validators import validate_ip_address


class ValidateIPAddressTestCase(TestCase):

    def test_validate_valid_ip_address(self):
        values = ['127.0.0.1', '1.1.1.1', '255.0.0.0', '0.0.0.0', 'fe80::1', '::1', '1:2:3:4:5:6:7:8']
        for value in values:
            self.assertEqual(validate_ip_address(value), value)

    def test_validate_invalid_ip_address(self):
        values = ['256.1.1.1', '25.1.1.', '25,1,1,1', '25.1 .1.1', '1:2', '::zzz', '12345::']
        for value in values:
            self.assertRaises(ValidationError, validate_ip_address, value)



# -*- coding: utf-8 -*-

from django.conf.urls import include, url
from django.contrib.auth.models import User
from django.test import override_settings
from rest_framework import status
from rest_framework.reverse import reverse

from rest_framework.test import APITestCase
from rest_framework.routers import DefaultRouter
from rest_framework.serializers import ModelSerializer
from rest_framework.viewsets import ModelViewSet

from timberjack.documents import ObjectAccessLog, READ, CREATE, UPDATE, DELETE
from timberjack.compat.rest_framework import mixins


class UserSerializer(ModelSerializer):
    class Meta:
        model = User
        fields = '__all__'


class UserViewSet(mixins.AccessLogModelViewMixin, ModelViewSet):
    write_admin_log = True
    serializer_class = UserSerializer
    queryset = User.objects.all()


router = DefaultRouter()
router.register(r'users', viewset=UserViewSet)
urlpatterns = [
    url(r'^', include(router.urls))
]


@override_settings(ROOT_URLCONF='tests.test_rest_framework_mixin')
class AccessLogModelViewMixinTestCase(APITestCase):

    def setUp(self):
        self.user = User.objects.create_user('testuser', 'testuser@example.com', 'test123.')
        self.client.login(username=self.user.username, password='test123.')

    def test_get_object_is_logged(self):
        response = self.client.get(reverse('user-detail', kwargs={'pk': self.user.pk}))
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        instance = ObjectAccessLog.objects.filter(action_flag=READ).order_by('-timestamp').first()
        self.assertEqual(instance.get_content_object(), self.user)

    def test_post_object_is_logged(self):
        response = self.client.post(reverse('user-list'), data={
            'username': 'another-user',
            'email': 'another-user@example.com',
            'password': 'test123.'
        }, format='json')
        self.assertEqual(response.status_code, status.HTTP_201_CREATED)

        instance = ObjectAccessLog.objects.filter(action_flag=CREATE).order_by('-timestamp').first()
        self.assertEqual(instance.get_content_object(), User.objects.get(username='another-user'))

    def test_patch_object_is_logged(self):
        response = self.client.patch(reverse('user-detail', kwargs={'pk': self.user.pk}), data={
            'username': 'changed-user'
        }, format='json')
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        instance = ObjectAccessLog.objects.filter(action_flag=UPDATE).order_by('-timestamp').first()
        self.assertEqual(instance.get_content_object(), self.user)

    def test_put_object_is_logged(self):
        response = self.client.put(reverse('user-detail', kwargs={'pk': self.user.pk}), data={
            'username': 'changed-user',
            'email': 'changed-user@example.com',
            'password': 'test123.'
        }, format='json')
        self.assertEqual(response.status_code, status.HTTP_200_OK)

        instance = ObjectAccessLog.objects.filter(action_flag=UPDATE).order_by('-timestamp').first()
        self.assertEqual(instance.get_content_object(), self.user)

    def test_delete_object_is_logged(self):
        user = User.objects.create_user('delete-me', 'delete-me@example.com', 'test123.')
        response = self.client.delete(reverse('user-detail', kwargs={'pk': user.pk}))
        self.assertEqual(response.status_code, status.HTTP_204_NO_CONTENT)

        instance = ObjectAccessLog.objects.filter(action_flag=DELETE).order_by('-timestamp').first()
        try:
            instance.get_content_object()
            self.fail('Did not fail when trying to get deleted instance')
        except User.DoesNotExist as e:
            self.assertEqual(str(e), 'User matching query does not exist.')



# -*- coding: utf-8 -*-

import json
import logging

from django.contrib.admin.models import LogEntry
from django.utils import timezone
from django.utils.encoding import smart_text
from django.utils.text import get_text_list
from django.utils.translation import ugettext, ugettext_lazy as _

from mongoengine import *
from mongoengine.queryset import QuerySet

from timberjack.constants import CREATE, READ, UPDATE, DELETE
from timberjack.fields import ContentTypeField, UserPKField
from timberjack.validators import validate_ip_address

LOG_LEVEL = (
    (0, _('NOTSET')),
    (10, _('DEBUG')),
    (20, _('INFO')),
    (30, _('WARNING')),
    (40, _('ERROR')),
    (50, _('CRITICAL'))
)

logger = logging.getLogger('timberjack')


class ObjectAccessLogQuerySet(QuerySet):

    def log_action(self, user_pk, content_type, object_pk, object_repr,
                   action_flag, message, level=0, ip_address=None, write_admin_log=False):
        if isinstance(message, list):
            message = json.dumps(message)
        return self._document(
            user_pk=user_pk,
            content_type=content_type,
            object_pk=object_pk,
            object_repr=object_repr[:200],
            action_flag=action_flag,
            message=message,
            level=level,
            ip_address=ip_address
        ).save(write_admin_log=write_admin_log)


class ObjectAccessLog(Document):

    meta = {
        'queryset_class': ObjectAccessLogQuerySet
    }

    message = StringField(default='')
    action_flag = IntField(min_value=1, max_value=4, required=True)
    level = IntField(choices=LOG_LEVEL, default=0)
    object_pk = DynamicField(required=True)
    content_type = ContentTypeField(required=True)
    object_repr = StringField(max_length=200, required=True)
    user_pk = UserPKField(default=None)
    ip_address = StringField(validation=validate_ip_address)
    admin_log_pk = IntField(default=None)
    referrer = ReferenceField('self', default=None)
    timestamp = DateTimeField(required=True, default=timezone.now)

    def __repr__(self):
        return smart_text(self.timestamp)

    def __str__(self):
        if self.is_create_action:
            return ugettext('Added "%(object)s.') % {'object': self.object_repr}
        elif self.is_update_action:
            return ugettext('Changed "%(object)s - %(changes)s"') % {
                'object': self.object_repr,
                'changes': self.get_change_message()
            }
        elif self.is_delete_action:
            return ugettext('Deleted "%(object)s."') % {'object': self.object_repr}
        elif self.is_read_action:
            return ugettext('Read "%(object)s."') % {'object': self.object_repr}

        return ugettext('ObjectAccessLog Object')

    @property
    def is_create_action(self):
        return self.action_flag == CREATE

    @property
    def is_read_action(self):
        return self.action_flag == READ

    @property
    def is_update_action(self):
        return self.action_flag == UPDATE

    @property
    def is_delete_action(self):
        return self.action_flag == DELETE

    def get_log_message(self):
        """
        (Copied from `django.contrib.admin.models.LogEntry.get_change_message()`)

        If self.change_message is a JSON structure, interpret it as a change
        string, properly translated.
        """
        if self.message and self.message[0] == '[':
            try:
                message = json.loads(self.message)
            except ValueError:
                return self.message
            messages = []
            for sub_message in message:
                if 'added' in sub_message:
                    if sub_message['added']:
                        sub_message['added']['name'] = ugettext(sub_message['added']['name'])
                        messages.append(ugettext('Added {name} "{object}".').format(**sub_message['added']))
                    else:
                        messages.append(ugettext('Added.'))

                elif 'changed' in sub_message:
                    sub_message['changed']['fields'] = get_text_list(
                        sub_message['changed']['fields'], ugettext('and')
                    )
                    if 'name' in sub_message['changed']:
                        sub_message['changed']['name'] = ugettext(sub_message['changed']['name'])
                        messages.append(ugettext('Changed {fields} for {name} "{object}".').format(
                            **sub_message['changed']
                        ))
                    else:
                        messages.append(ugettext('Changed {fields}.').format(**sub_message['changed']))

                elif 'deleted' in sub_message:
                    sub_message['deleted']['name'] = ugettext(sub_message['deleted']['name'])
                    messages.append(ugettext('Deleted {name} "{object}".').format(**sub_message['deleted']))

                elif 'read' in sub_message:
                    if sub_message['read']:
                        sub_message['read']['name'] = ugettext(sub_message['read']['name'])
                        messages.append(ugettext('Read {name} "{object}".').format(**sub_message['read']))
                    else:
                        messages.append(ugettext('Read.'))

            message = ' '.join(msg[0].upper() + msg[1:] for msg in messages)
            return message or ugettext('No fields changed.')
        else:
            return self.message

    def get_admin_log_object(self):
        """
        If saved with an `admin_log_pk` attribute, look up
        and return the corresponding object.
        """
        if not self.admin_log_pk:
            return None
        try:
            return LogEntry.objects.get(pk=self.admin_log_pk)
        except LogEntry.DoesNotExist:
            return None

    def get_content_object(self):
        return self.content_type.get_object_for_this_type(pk=self.object_pk)

    def save(self, *args, **kwargs):
        if kwargs.get('write_admin_log', False) is True:
            if self.is_read_action:
                logger.debug('Read actions are not written to the `admin.LogEntry` table due '
                             'to missing support for read actions.')
            elif not self.user_pk:
                logger.debug('Action cannot be written to the `admin.LogEntry` table due '
                             'to missing `user_id` value.')
            else:
                self.admin_log_pk = LogEntry.objects.create(user_id=self.user_pk, content_type_id=self.content_type.pk,
                                                            object_id=self.object_pk,
                                                            object_repr=repr(self.content_type.get_object_for_this_type(
                                                                pk=self.object_pk))[:200], action_flag=self.action_flag,
                                                            change_message=self.message).pk
        return super(ObjectAccessLog, self).save(*args, **kwargs)



# -*- coding: utf-8 -*-

from mongoengine.connection import DEFAULT_CONNECTION_NAME

DEFAULT_ALIAS = DEFAULT_CONNECTION_NAME  # pragma: no cover

# We want to log CRUD actions.
# (keep numerical values compatible with admin.LogEntry model)
CREATE = 1
UPDATE = 2
DELETE = 3
READ = 4



# -*- coding: utf-8 -*-

from django.contrib.admin.models import ADDITION, CHANGE, DELETION, LogEntry
from django.contrib.auth import get_user_model
from django.contrib.contenttypes.models import ContentType
from django.test import TestCase

from timberjack.constants import CREATE, UPDATE, DELETE, READ
from timberjack.documents import LOG_LEVEL, ObjectAccessLog

USER_MODEL = get_user_model()


class ModelConstantsTestCase(TestCase):

    def test_crud_action_values(self):
        self.assertEqual(CREATE, 1)
        self.assertEqual(UPDATE, 2)
        self.assertEqual(DELETE, 3)
        self.assertEqual(READ, 4)

    def test_crud_action_values_django_compatibility(self):
        self.assertEqual(CREATE, ADDITION)
        self.assertEqual(UPDATE, CHANGE)
        self.assertEqual(DELETE, DELETION)

    def test_log_level_values(self):
        log_levels = ['NOTSET', 'DEBUG', 'INFO', 'WARNING', 'ERROR', 'CRITICAL']
        for i, values in enumerate(LOG_LEVEL):
            num, value = values
            self.assertEqual(i * 10, num)
            self.assertEqual(value, log_levels[i])


class ObjectAccessLogTestCase(TestCase):

    def setUp(self):
        self.user = USER_MODEL.objects.create_user(username='test@example.com', password='test123.')
        self.ctype = ContentType.objects.get_for_model(self.user)

    def test_document_initialize(self):
        instance = ObjectAccessLog(user_pk=self.user.pk, content_type=self.ctype, object_pk=self.user.pk,
                                   object_repr=repr(self.user), action_flag=1, message='test message')
        self.assertIsInstance(instance, ObjectAccessLog)

    def test_document_properties(self):
        instance = ObjectAccessLog(user_pk=self.user.pk, content_type=self.ctype, object_pk=self.user.pk,
                                   object_repr=repr(self.user), action_flag=1, message='test message',
                                   level=0)
        self.assertTrue(instance.is_create_action)

        instance.action_flag = 2
        self.assertTrue(instance.is_update_action)

        instance.action_flag = 3
        self.assertTrue(instance.is_delete_action)

        instance.action_flag = 4
        self.assertTrue(instance.is_read_action)

    def test_document_get_log_message(self):
        instance = ObjectAccessLog(user_pk=self.user.pk, content_type=self.ctype, object_pk=self.user.pk,
                                   object_repr=repr(self.user), action_flag=1, message='test message',
                                   level=0)
        self.assertEqual(instance.get_log_message(), 'test message')

        instance.message = ''
        self.assertEqual(instance.get_log_message(), '')

    def test_document_write_admin_log(self):
        instance = ObjectAccessLog(user_pk=self.user.pk, content_type=self.ctype, object_pk=self.user.pk,
                                   object_repr=repr(self.user), action_flag=1, message='test message',
                                   level=0)
        instance.save(write_admin_log=True)

        self.assertIsInstance(instance.get_admin_log_object(), LogEntry)
        self.assertEqual(instance.get_admin_log_object(), LogEntry.objects.get(pk=instance.admin_log_pk))

    def test_document_referrer(self):
        instance1 = ObjectAccessLog(user_pk=self.user.pk, content_type=self.ctype, object_pk=self.user.pk,
                                    object_repr=repr(self.user), action_flag=1, message='message 1',
                                    level=0)
        instance1.save()

        instance2 = ObjectAccessLog(user_pk=self.user.pk, content_type=self.ctype, object_pk=self.user.pk,
                                    object_repr=repr(self.user), action_flag=1, message='message 2',
                                    level=0, referrer=instance1)
        instance2.save()

        self.assertEqual(instance1, instance2.referrer)


class ObjectAccessQuerySetTestCase(TestCase):

    def setUp(self):
        self.user = USER_MODEL.objects.create_user(username='test@example.com', password='test123.')
        self.ctype = ContentType.objects.get_for_model(self.user)

    def test_queryset_log_action(self):
        instance = ObjectAccessLog.objects.log_action(user_pk=self.user.pk, content_type=self.ctype,
                                                      object_pk=self.user.pk, object_repr=repr(self.user),
                                                      action_flag=1, message='test message')
        self.assertIsInstance(instance, ObjectAccessLog)
